#include "haar_detector.h"

haar_detector::haar_detector(human_parts_class* parts_class, IplImage* gray_image) {
	
	this->parts_class = parts_class;
	this->gray_image  = gray_image;
}

bool haar_detector::run_detection() {
	
	int i;
	bool found_something = false;
	
	//DETECT FRONTAL FACES
	found_something = this->detect("haarcascades/frontalface_alt.xml", "frontal face") || found_something;
	
	//DETECT DYNAMIC FACES
	//found_something = this->detect("haarcascades/dynamic_faces.xml", "dynamic face") || found_something;
	
	//DETECT POORLY TRAINED DYNAMIC FACES
	//found_something = this->detect("haarcascades/dynamic_faces--poorly_trained.xml", "dynamic face (POORLY TRAINED)") || found_something;

	//DETECT FULL-BODIES
	found_something = this->detect("haarcascades/fullbody.xml", "fullbody") || found_something;

	//DETECT LOWER-BODIES
	//found_something = this->detect("haarcascades/lowerbody.xml", "lower body") || found_something;

	//DETECT UPPER-BODIES
	found_something = this->detect("haarcascades/upperbody.xml", "upper body") || found_something;
	
	//DETECT LIMBS
	//found_something = this->detect("haarcascades/limbs_cascade.xml", "limbs") || found_something;
	
	//DETECT V'S
	//found_something = this->detect("haarcascades/vs_cascade.xml", "body V's") || found_something;
	
	return found_something;
}

bool haar_detector::detect(const char* cascade_file, const char* type) {
	
	//LOAD CASCADE FILE
	CvHaarClassifierCascade* cascade = (CvHaarClassifierCascade*)cvLoad(cascade_file, 0, 0, 0);
	if (!cascade)
		throw cascade_file;
	
	//CREATE AND CLEAR MEMORY STORAGE
	CvMemStorage* storage = cvCreateMemStorage(0);
	cvClearMemStorage(storage);
	
	//RUN HAAR ALGORITHM AND RETRIEVE SEQUENCE OF PARTS
	CvSeq* detected_objects = cvHaarDetectObjects(this->gray_image,
	                                              cascade, 
	                                              storage, 
	                                              1.1, // SCALE FACTOR
	                                              4,   // MIN NEIGHBOR RECTS
	                                              CV_HAAR_DO_CANNY_PRUNING,
	                                              cvSize(20, 20) //MIN WINDOW SIZE
	                                              );
	
	//STORE ALL DETECTED OBJECTS' BOUNDING BOXES
	if (detected_objects->total) {
		
		CvRect* this_object_box;
		int x1, y1, x2, y2;
		int i;
		
		for (i=0; i<detected_objects->total; i++) {
			
			//RETRIEVE BOUNDING BOX OF THE DETECTED OBJECT
			this_object_box = (CvRect*)cvGetSeqElem(detected_objects, i);
			
			x1 = this_object_box->x;
			y1 = this_object_box->y;
			x2 = this_object_box->x + this_object_box->width;
			y2 = this_object_box->y + this_object_box->height;
			
			cout <<"Found a \""<<type<<"\" at ("<<x1<<", "<<y1<<"), ("<<x2<<", "<<y2<<")"<<endl;
			
			this->parts_class->create_box(x1, y1, x2, y2);
		}
		
		return true;
		
	} else {
		
		return false;
	}
}
