#include "human_detector.h"

human_detector::human_detector(const char* image_file) {
	
	this->showing_display_window = false;
	
	//LOAD THE SUPPLIED IMAGE FILE INTO A CV IMAGE 
	if ((this->orig_image = cvLoadImage(image_file, 1)) == 0)		
		throw "Unable to load supplied image file!\nThe supplied file may not exist or may be of an invalid type.";
	
	//GENERATE GRAYSCALE VERSION OF IMAGE FOR USE IN DETECTION METHODS
	this->orig_image_size = cvSize(orig_image->width, orig_image->height);
	this->gray_image = cvCreateImage((CvSize)orig_image_size, /*COLOR DEPTH*/ 8, /*CHANNELS PER PIXEL*/ 1);
	cvCvtColor(this->orig_image, this->gray_image, CV_BGR2GRAY);
	
	//GENERATE DISPLAY IMAGE
	this->display_image = cvCloneImage(this->orig_image);
}

human_detector::human_detector(IplImage* orig_image) {
	
	this->showing_display_window = false;
	
	//LOAD THE SUPPLIED IMAGE FILE INTO A CV IMAGE 
	this->orig_image = orig_image;
	
	//GENERATE GRAYSCALE VERSION OF IMAGE FOR USE IN DETECTION METHODS
	this->orig_image_size = cvSize(orig_image->width, orig_image->height);
	this->gray_image = cvCreateImage((CvSize)orig_image_size, /*COLOR DEPTH*/ 8, /*CHANNELS PER PIXEL*/ 1);
	cvCvtColor(this->orig_image, this->gray_image, CV_BGR2GRAY);
	
	//GENERATE DISPLAY IMAGE
	this->display_image = cvCloneImage(this->orig_image);
}

human_detector::~human_detector() {
	
	//RELEASE IMAGE HEADERS AND DATA
	cvReleaseImage(&this->orig_image);
	cvReleaseImage(&this->gray_image);
	cvReleaseImage(&this->display_image);
}

bool human_detector::detect() {
	
	/* RUN VARIOUS DETECTION METHODS */
	
	this->detected_parts = new human_parts_class();
	
	//HAAR METHOD
	haar_detector* haar = new haar_detector(this->detected_parts, this->gray_image);
	haar->run_detection();
	this->update_display_window();
	
	//V&P METHOD
	
	
	//EDGE DETECTION METHOD
	
	
	/* CLUSTER DETECTED PARTS TO IDENTIFY POTENTIAL PEOPLE */
	
	
}

void human_detector::show_display_window() {
	
	// OPEN DISPLAY WINDOW TO SHOW RESULTS
	cvNamedWindow("human_detector_display", 1);
	
	//DISPLAY ORIGINAL IMAGE IN DISPLAY WINDOW
	cvShowImage("human_detector_display", this->display_image);
	
	this->showing_display_window = true;
}

void human_detector::hide_display_window() {
	
	cvDestroyWindow("human_detector_display");
	
	this->showing_display_window = false;
}

void human_detector::update_display_window() {
	
	//IF NOT SHOWING DISPLAY WINDOW, DON'T DO ANYTHING
	if (!this->showing_display_window)
		return;
	
	int i;
	
	//DISPLAY INDIVIDUAL PARTS
	if (this->detected_parts->get_num_parts()) {
		
		struct human_part_box *this_box;
		for (i=0; i<this->detected_parts->get_num_parts(); i++) {
			
			this_box = this->detected_parts->get_part(i);
			
			//cout << "Part " << i << " at ("<< this_box->x1 << ", " << this_box->y1 << "), (" << this_box->x2 << ", " << this_box->y2 << ")..." << endl;
			
			CvPoint this_part_uleft;
			this_part_uleft.x  = this_box->x1;
			this_part_uleft.y  = this_box->y1;
			
			CvPoint this_part_bright;
			this_part_bright.x = this_box->x2;
			this_part_bright.y = this_box->y2;
			
			//DRAW RECT ON DISPLAY IMAGE AROUND THIS PART
			cvRectangle(this->display_image,
			            this_part_uleft,
			            this_part_bright,
			            (CvScalar){{255, 255, 255}},
			            1, //LINE THICKNESS
			            CV_AA //ANTI-ALIASED LINE
			            );
			
			cvShowImage("human_detector_display", display_image);
		}
	}
}
